from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from sqlalchemy import insert, select
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import RoleModel, UserModel, UserRoleAssociation


class UserRepository(SQLAlchemyAsyncRepository):
    model_type = UserModel

    async def create_admin(
        self,
        user: UserModel,
    ) -> UserModel:
        user = await self.add(user, auto_commit=True)
        return await self.add_user_role(
            user,
            "admin",
        )

    async def add_user_role(
        self,
        user: UserModel,
        role_name: str,
    ) -> UserModel:
        role = await self.session.scalar(select(RoleModel).where(RoleModel.name == role_name))
        await self.session.execute(insert(UserRoleAssociation).values(user_id=user.id, role_id=role.id))

        await self.session.commit()
        return user


async def provide_user_repository(db_session: AsyncSession) -> UserRepository:
    return UserRepository(session=db_session)



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy.orm import Mapped, mapped_column, relationship

from .auth_association_models import RolePermissionAssociation, UserRoleAssociation


class RoleModel(UUIDAuditBase):
    __tablename__ = "roles"

    name: Mapped[str] = mapped_column(unique=True, nullable=False)
    users = relationship(
        "UserModel",
        secondary="users_roles",
        back_populates="roles",
        lazy="selectin",
    )
    permissions = relationship(
        "PermissionModel",
        secondary="roles_permissions",
        back_populates="roles",
        lazy="selectin",
    )



from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import RoleModel


class RoleRepository(SQLAlchemyAsyncRepository):
    model_type = RoleModel


async def provide_role_repository(db_session: AsyncSession) -> RoleRepository:
    return RoleRepository(session=db_session)



from __future__ import annotations

from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from advanced_alchemy.service import SQLAlchemyAsyncRepositoryService
from lab.auth.models import PermissionModel

__all__ = ("PermissionService",)


class PermissionService(SQLAlchemyAsyncRepositoryService[PermissionModel]):
    class Repository(SQLAlchemyAsyncRepository[PermissionModel]):
        model_type = PermissionModel

    repository_type = Repository



from litestar.plugins.sqlalchemy import (
    AsyncSessionConfig,
    SQLAlchemyAsyncConfig,
    SQLAlchemyPlugin,
)
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine

from ..settings import settings

session_config = AsyncSessionConfig(expire_on_commit=False)
sqlalchemy_config = SQLAlchemyAsyncConfig(
    connection_string=settings.database_uri,
    before_send_handler="autocommit",
    session_config=session_config,
)
alchemy = SQLAlchemyPlugin(config=sqlalchemy_config)

engine = create_async_engine(settings.database_uri, echo=False)
session_maker = async_sessionmaker(
    engine,
    expire_on_commit=False,
    class_=AsyncSession,
)



from __future__ import annotations

from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from advanced_alchemy.service import SQLAlchemyAsyncRepositoryService
from lab.auth.models import UserModel

__all__ = ("UserService",)


class UserService(SQLAlchemyAsyncRepositoryService[UserModel]):
    class Repository(SQLAlchemyAsyncRepository[UserModel]):
        model_type = UserModel

    repository_type = Repository
    match_fields = ["email"]



import os

import yaml
from click import Group
from litestar.plugins import CLIPluginProtocol

from ..core.cli import coro
from ..core.database import session_maker
from .auth_core import hash_password
from .models import PermissionModel, RoleModel, UserModel
from .repositories.permission_repository import provide_permission_repository
from .repositories.role_repository import provide_role_repository
from .repositories.user_repository import provide_user_repository
import click


class CLIPlugin(CLIPluginProtocol):
    def on_cli_init(self, cli: Group) -> None:
        @cli.group(help="Manage auth, load data with ``load`` command")
        @click.version_option(prog_name='mycli')
        def auth():
            ...

        @auth.command(help="Load auth initial data")
        @coro
        async def load():
            async with session_maker() as session:
                permission_repository = await provide_permission_repository(session)
                role_repository = await provide_role_repository(session)
                user_repository = await provide_user_repository(session)

                await permission_repository.delete_where(auto_commit=True)
                await role_repository.delete_where(auto_commit=True)
                await user_repository.delete_where(auto_commit=True)

                with open(
                        os.path.join(os.path.dirname(os.path.abspath(__file__)), "./fixtures/auth.yaml"), "r"
                ) as stream:
                    data_loaded = yaml.safe_load(stream)
                    permissions = await permission_repository.add_many(
                        [
                            PermissionModel(app=app, name=name)
                            for app, names in data_loaded.get("permissions").items()
                            for name in names
                        ],
                        auto_commit=True,
                    )

                    roles = []
                    for name, permission_values in data_loaded.get("roles").items():
                        role = RoleModel(name=name)
                        role_permission_filter = {
                            app: {name for name in names}
                            for app, names in permission_values.items()
                        }
                        for permission in permissions:
                            if permission.name in role_permission_filter.get(permission.app):
                                role.permissions.append(permission)
                        roles.append(role)

                    await role_repository.add_many(roles, auto_commit=True)
                    users = []

                    for user_data in data_loaded.get("users"):
                        email = user_data.get("email")
                        users_roles = {role for role in user_data.get("roles")}

                        user = UserModel(email=email, password=hash_password("password"))
                        for role in roles:
                            if role.name in users_roles:
                                user.roles.append(role)
                        users.append(user)
                    await user_repository.add_many(users, auto_commit=True)

                click.secho("Success loaded init data", fg="green")



import asyncio
from functools import wraps

from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine

from ..settings import settings

engine = create_async_engine(settings.database_uri, echo=False)
session_maker = async_sessionmaker(
    engine,
    expire_on_commit=False,
    class_=AsyncSession,
)

def coro(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        return asyncio.run(f(*args, **kwargs))

    return wrapper




from litestar.openapi import OpenAPIConfig
from litestar.openapi.plugins import ScalarRenderPlugin
from litestar.openapi.spec import Components, SecurityScheme


def configure_open_api_config(title: str, version: str, path: str="/api/docs"):
    return OpenAPIConfig(
        components=[
            Components(
                security_schemes={
                    "JWT": SecurityScheme(
                        type="http",
                        scheme="Bearer",
                        name="Authorization",
                        security_scheme_in="cookie",
                        bearer_format="JWT",
                        description="Authorization",
                    )
                }
            )
        ],
        title=title,
        version=version,
        path=path,
        root_schema_site="elements",
        render_plugins=[ScalarRenderPlugin()],
    )




from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_file=".env", env_file_encoding="utf-8")
    database_uri: str = Field(
        default="postgresql+asyncpg://user:password@localhost:5432/database"
    )
    secret_key: str = Field(default="Secret_Key")

settings = Settings()



from advanced_alchemy.base import UUIDAuditBase
from sqlalchemy import Column, ForeignKey, Table
from uuid import UUID


from sqlalchemy import UniqueConstraint
from sqlalchemy.orm import Mapped, mapped_column, relationship



class UserRoleAssociation(UUIDAuditBase):
    __tablename__ = "users_roles"

    user_id: Mapped[UUID] = mapped_column(ForeignKey("users.id"), primary_key=True)
    role_id: Mapped[UUID] = mapped_column(ForeignKey("roles.id"), primary_key=True)


class RolePermissionAssociation(UUIDAuditBase):
    __tablename__ = "roles_permissions"

    role_id: Mapped[UUID] = mapped_column(ForeignKey("roles.id"), primary_key=True)
    permission_id: Mapped[UUID] = mapped_column(ForeignKey("permissions.id"), primary_key=True)



from litestar import Router

from .account_controller import AccountController
from .role_controller import RoleController
from .user_controller import UserController

auth_router = Router(path="/api/auth", route_handlers=[RoleController, UserController, AccountController])
